#include <zephyr/kernel.h>
#include "TaskMain.h"
#include "TaskLED.h"
#include "TaskBeep.h"
#include "TaskButton.h"
#include "TaskBLE.h"
#include <zephyr/logging/log.h>
#include "main.h"

LOG_MODULE_REGISTER(task_main, LOG_LEVEL_INF);

#define MAIN_STACK_SIZE 1024
#define MAIN_PRIORITY   0
#define MAIN_MSGQ_LEN   16

K_THREAD_STACK_DEFINE(main_stack_area, MAIN_STACK_SIZE);
static struct k_thread main_thread;

K_MSGQ_DEFINE(main_msgq, sizeof(main_event_t), MAIN_MSGQ_LEN, 4);

static void Task_Main(void *a, void *b, void *c);
static void Main_Init(TaskMain_ctx_t *ctx);
static void Main_Off_State(const main_event_t *evt, TaskMain_ctx_t *ctx);
static void Main_On_State(const main_event_t *evt, TaskMain_ctx_t *ctx);

static void short_press_feedback(uint8_t led_index)
{
    led_msg_t led_msg = 
    {
        .cmd = LED_SHORT_PRESS,
        .led_index = led_index,
        .repeat_count = 1,
        .wait_for_previous = false,
    };
    TaskLED_SendCommand(&led_msg);

    beep_msg_t beep_msg = 
    {
        .cmd = BEEP_SHORT_PRESS,
        .repeat_count = 1,
        .wait_for_previous = false,
    };
    TaskBeep_SendCommand(&beep_msg);
}

// Externí API pro jiné tasky (např. TaskButton): posílání eventu
int TaskMain_PutEvent(const main_event_t *evt)
{
    return k_msgq_put(&main_msgq, evt, K_FOREVER);
}

void TaskMain_Start(void)
{
    k_thread_create(
        &main_thread,
        main_stack_area,
        MAIN_STACK_SIZE,
        Task_Main,
        NULL, NULL, NULL,
        MAIN_PRIORITY, 0, K_NO_WAIT
    );
    k_thread_name_set(&main_thread, "TaskMain");
}

static void Main_Init(TaskMain_ctx_t *ctx)
{
    // Inicializace potřebných komponent
    TaskLED_Start();
    TaskBeep_Start();
    TaskButton_Start();
    // TaskBLE_Start();

    led_msg_t msg = 
    {
        .cmd = LED_SYSTEM_ON,
        .led_index = 1,
        .repeat_count = INFINITY_BLINK_T, // nekonečné blikání
        .wait_for_previous = false
    };
    TaskLED_SendCommand(&msg);

    beep_msg_t msg_beep = 
    {
        .cmd = BEEP_CMD_SYSTEM_ON,
        .repeat_count = 1
    };
    TaskBeep_SendCommand(&msg_beep);

    ctx->state = SYSTEM_STATE_ON;
}

static void Main_Off_State(const main_event_t *evt, TaskMain_ctx_t *ctx)
{
    switch (evt->type)
    {
        case MAIN_EVENT_BTN_PRESSED_EDGE:
        {
            short_press_feedback(0);
            break;
        }
        case MAIN_EVENT_BTN_LONG_PRESS:
        {
            ctx->state = SYSTEM_STATE_ON;

            led_msg_t led_on = 
            {
                .cmd = LED_SYSTEM_ON,
                .led_index = 1,
                .repeat_count = INFINITY_BLINK_T, // stejné jako v Main_Init!
                .wait_for_previous = false
            };
            TaskLED_SendCommand(&led_on);

            beep_msg_t beep = 
            {
                .cmd = BEEP_CMD_SYSTEM_ON,
                .repeat_count = 1
            };
            TaskBeep_SendCommand(&beep);

            LOG_INF("System turned ON");
            break;
        }
        default:
            break;
    }
}

static void Main_On_State(const main_event_t *evt, TaskMain_ctx_t *ctx)
{
    static uint8_t cnt;
    switch (evt->type)
    {
        case MAIN_EVENT_BTN_PRESSED_EDGE:
        {
            short_press_feedback(1);

            // Po krátkém bliku obnov SYSTEM_ON pattern (čeká na dokončení předchozího)
            led_msg_t led_sys = 
            {
                .cmd = LED_SYSTEM_ON,
                .led_index = 1,
                .repeat_count = INFINITY_BLINK_T,
                .wait_for_previous = true
            };
            TaskLED_SendCommand(&led_sys);

            beep_msg_t beep1 = 
            {
                .cmd = BEEP_TONE_ZONE1 + cnt,
                .repeat_count = 1,
                .wait_for_previous = false
            };
            TaskBeep_SendCommand(&beep1);

            cnt++;
            if (cnt == 7) cnt = 0;
            break;
        }

        case MAIN_EVENT_BTN_LONG_PRESS:
        {
            ctx->state = SYSTEM_STATE_OFF;

            led_msg_t led_off = 
            {
                .cmd = LED_CMD_OFF,
                .led_index = 1,
                .repeat_count = 1,
                .wait_for_previous = false
            };
            TaskLED_SendCommand(&led_off);

            beep_msg_t beep = 
            {
                .cmd = BEEP_CMD_SYSTEM_OFF,
                .repeat_count = 1
            };
            TaskBeep_SendCommand(&beep);

            LOG_INF("System turned OFF");
            break;
        }
        case MAIN_EVENT_BTN_SHORT_PRESS:
            // Pokud bude potřeba, doplň další reakce.
            break;
        case MAIN_EVENT_BTN_EXTRA:
            break;
        default:
            break;
    }
}

// ----------- Vlastní Control/Main vlákno --------------------

static void Task_Main(void *a, void *b, void *c)
{
    TaskMain_ctx_t ctx;
    main_event_t evt;

    Main_Init(&ctx);

    while (1)
    {
        log_stack_usage("TaskMain");

        #if defined(CONFIG_STACK_WARN_THRESHOLD)
        LOG_INF("Threshold is %d B", CONFIG_STACK_WARN_THRESHOLD);
        #endif

        if (k_msgq_get(&main_msgq, &evt, K_FOREVER) == 0)
        {
            switch (ctx.state)
            {
                case SYSTEM_STATE_OFF:
                    Main_Off_State(&evt, &ctx);
                    break;
                case SYSTEM_STATE_ON:
                    Main_On_State(&evt, &ctx);
                    break;
                // Další stavy např. FACTORY apod.
                default:
                    break;
            }
        }
    }
}
